{% load follow_tags %}
<form action="{% follow_url object %}" method="post">
	{% csrf_token %}
	{% if request.user|is_following:object %}
		<input type="submit" value="Unfollow" />
	{% else %}
		<input type="submit" value="Follow" />
	{% endif %}
</form>